from asyncio import Event, create_task
from uuid import uuid4

from ap2p.node import Node

SYNC_REQUEST = b"SYNC"


class SyncRequestHandler:
    def __init__(self, uid, handlers=[]):
        self.uid = uid
        self.handlers = handlers
        self.contents = []

    def update(self, content, writer):
        self.contents.append((content, writer))


class SyncMessagingNode(Node):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.sync_request_events = {}
        self.sync_request_responses = {}

    async def ping_waiter(self, uid, data):
        await self.sync_requests[uid].wait()
        assert uid in self.sync_request_responses
        assert self.sync_request_responses[uid] == b"PONG " + data
        del self.sync_requests[uid]
        del self.sync_request_responses[uid]

    async def connect_to_peer(self, peer):
        result = await super().connect_to_peer(peer)
        uuid = uuid4().hex
        self.sync_requests[uuid] = Event()
        create_task(self.ping_waiter(uuid))
        return result
